# -*- coding: utf-8 -*-

from collective.anysurfer.interfaces import ILayerSpecific
from collective.folderishtypes.interfaces import IFolderishType
from cpskin.core.interfaces import ICPSkinSettings
from Acquisition import aq_parent
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.utils import isDefaultPage
from Products.Five import BrowserView
from zope.component import getUtility
from plone import api


class UtilsView(BrowserView):

    def is_gdpr(self):
        return api.portal.get_registry_record(
            "imio.gdpr.interfaces.IGDPRSettings.is_text_ready", default=False
        )

    def has_enabled_accessibilty_link_in_footer(self):
        return api.portal.get_registry_record(
            "cpskin.core.interfaces.ICPSkinSettings.enable_accessibility_link_in_footer",
            default=False,
        )

    def edit_anysurfer_settings(self):
        if api.user.is_anonymous():
            return False
        current_user = api.user.get_current()
        roles = api.user.get_roles(username=current_user.getUserName())
        if "Manager" not in roles and "Site Administrator" not in roles:
            return False
        if not ILayerSpecific.providedBy(self.request):
            return False
        if bool(self.request.get("change_link_visibility", False)) is True:
            registry = getUtility(IRegistry)
            records = registry.records
            # cpskin.core.interfaces.ICPSkinSettings
            value = not records["cpskin.core.interfaces.ICPSkinSettings.enable_accessibility_link_in_footer"].value
            api.portal.set_registry_record(
                "enable_accessibility_link_in_footer", value, interface=ICPSkinSettings
            )
            self.request.response.redirect(self.context.absolute_url())
            self.request.response.setHeader('Cache-Control', 'no-cache, no-store')
        return True

    def has_folderish_default(self):
        if not IFolderishType.providedBy(self.context):
            return False
        return isDefaultPage(self.context, self.request)

    def convert_default_url(self, url, to_parent=False):
        """Converts url from default view url to parent or vice versa
        """
        obj_url = self.context.absolute_url()
        parent_url = aq_parent(self.context).absolute_url()
        if to_parent and obj_url in url:
            return url.replace(obj_url, parent_url)
        if not to_parent and obj_url not in url:
            return url.replace(parent_url, obj_url)
        return url



# -*- coding: utf-8 -*-
from plone.app.event.browser.event_listing import EventListing as BaseEventListing


class EventListing(BaseEventListing):

    def __call__(self, *args, **kwargs):
        return



# -*- coding: utf-8 -*-

from collective.privacy.browser.consent import ConsentForm
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from Products.Five import BrowserView
from z3c.form import button
from zope.i18n import translate

import json


def get_all_consent_reasons(privacy_tool):
    for reason in privacy_tool.getAllReasons().values():
        if reason.lawful_basis.__name__ == "consent":
            yield reason


class PrivacyView(BrowserView):
    """ """

    def allow_iframes(self):
        self.request.response.setHeader("Content-type", "application/json")
        portal_privacy = api.portal.get_tool("portal_privacy")
        return json.dumps(portal_privacy.processingIsAllowed("show_genetic_embed"))

    def allow_languages(self):
        self.request.response.setHeader("Content-type", "application/json")
        portal_privacy = api.portal.get_tool("portal_privacy")
        return json.dumps(portal_privacy.processingIsAllowed("language_preference"))

    def accept_or_refuse_all(self):
        form = self.request.form
        came_from = form.get("came_from")
        if not came_from:
            came_from = api.portal.get_navigation_root(self.context).absolute_url()
        accept_all = True if "consent" in form else False
        privacy_tool = api.portal.get_tool("portal_privacy")
        for reason in get_all_consent_reasons(privacy_tool):
            if accept_all:
                privacy_tool.consentToProcessing(reason.__name__)
            else:
                privacy_tool.objectToProcessing(reason.__name__)
        self.request.response.redirect(came_from)
        return ""


class ConsentFormWithPolicy(ConsentForm):

    label = _(u"Cookies choice")
    id = "cookies-form"

    def update(self):
        super(ConsentFormWithPolicy, self).update()
        root = api.portal.get_navigation_root(self.context)
        current_lang = api.portal.get_current_language()[:2]
        policy_url = u"{}/@@cookies-view".format(root.absolute_url())
        description = _(
            u"Choose to opt in or out of cookies use.<br/>"
            u'Our <a href="${policy_url}">cookies policy</a> can help you choose.',
            mapping={u"policy_url": policy_url},
        )
        self.description = translate(description, target_language=current_lang)

    def _redirect(self):
        if "ajax_load" in self.request.form:
            # the form was loaded via an overlay, we need to redirect to
            # an existing page to close it
            portal = api.portal.get()
            self.request.response.redirect("{}/@@ok".format(portal.absolute_url()))
            return ""

    @button.buttonAndHandler(_(u"Save my choices"))
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        privacy_tool = self.context.portal_privacy
        for topic, answer in data.items():
            answer = answer == "Allowed"
            if answer:
                privacy_tool.consentToProcessing(topic)
            else:
                privacy_tool.objectToProcessing(topic)
        self.status = _(u"Your preferences have been saved.")
        self._redirect()

    @button.buttonAndHandler(_(u"Accept all"))
    def handleAcceptAll(self, action):
        privacy_tool = api.portal.get_tool("portal_privacy")
        for reason in get_all_consent_reasons(privacy_tool):
            privacy_tool.consentToProcessing(reason.__name__)
        self._redirect()

    @button.buttonAndHandler(_(u"Refuse all"))
    def handleRefuseAll(self, action):
        privacy_tool = api.portal.get_tool("portal_privacy")
        for reason in get_all_consent_reasons(privacy_tool):
            privacy_tool.objectToProcessing(reason.__name__)
        self._redirect()



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from plone.dexterity.interfaces import IDexterityContent
from plone.transformchain.interfaces import ITransform
from zope.component import adapter
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface


def replace_iframe(soup, message):
    """
    Change <iframe> attributes to make it work only if cookies have been accepted
    The iframe attributes are put back through JS (to avoid server caching)
    """
    tags = soup.findAll("iframe")
    for tag in tags:
        privacy_tag = soup.new_tag("div")
        privacy_tag.append(BeautifulSoup(message, "html.parser"))
        privacy_tag["class"] = "gdpr-iframe-message"
        tag["gdpr-src"] = tag["src"]
        tag["src"] = ""
        if tag.get("width"):
            tag["gdpr-width"] = tag["width"]
        if tag.get("height"):
            tag["gdpr-height"] = tag["height"]
        tag["width"] = tag["height"] = "0"
        if tag.get("class"):
            tag["class"].append("gdpr-iframe")
        else:
            tag["class"] = ["gdpr-iframe"]
        tag.insert_after(privacy_tag)


@implementer(ITransform)
@adapter(Interface, Interface)
class EmbedTransform(object):

    order = 1000000

    def __init__(self, published, request):
        self.published = published
        self.request = request
        self.message = None

    def applyTransform(self):
        site = getSite()
        if not site:
            return False
        if self.published is None:
            return False
        responseType = self.request.response.getHeader("content-type") or ""
        if not responseType.startswith("text/html") and not responseType.startswith(
            "text/xhtml"
        ):
            return False
        return True

    def prepareMessage(self):
        if self.message:
            return
        current_lang = api.portal.get_current_language()[:2]
        root = api.portal.get_navigation_root(self.published)
        if not IDexterityContent.providedBy(root):
            root = api.portal.get()
        consent_url = u"{}/@@consent".format(root.absolute_url())
        message = _(
            u"This feature requires cookies acceptation.<br/>"
            u'<a class="consent-link" href="${consent_url}">Review your cookies preferences</a>.',
            mapping={u"consent_url": consent_url},
        )
        self.message = translate(message, target_language=current_lang)

    def transformBytes(self, result, encoding):
        if not self.applyTransform():
            return result
        soup = BeautifulSoup(result, "lxml")
        self.prepareMessage()
        replace_iframe(soup, self.message)
        return str(soup)

    def transformUnicode(self, result, encoding):
        if not self.applyTransform():
            return result
        soup = BeautifulSoup(result, "lxml")
        self.prepareMessage()
        replace_iframe(soup, self.message)
        return str(soup)

    def transformIterable(self, result, encoding):
        if not self.applyTransform():
            return result
        self.prepareMessage()
        transformed = []
        for r in result:
            soup = BeautifulSoup(r, "lxml")
            replace_iframe(soup, self.message)
            transformed.append(str(soup))
        return transformed
